-------------------------------------------------------------------------------
--! @file       CA.vhd
--! @author     Ahmed Ferozpuri
--! @brief      ConstantAddition (CA) for PRIMATEs
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY CA IS
	GENERIC (V : INTEGER := 200;
				isINV : INTEGER := 0);
	PORT(	clk, enLFSR, selLFSR: IN std_logic;
			pn : IN std_logic_vector(1 DOWNTO 0);
			V_i : IN  std_logic_vector(V-1 DOWNTO 0);
			V_o : OUT  std_logic_vector(V-1 DOWNTO 0)
		);
END CA;

ARCHITECTURE behavioral OF CA IS

	SIGNAL LFSRInit, LFSRShift, LFSRIn, RC, V11 : std_logic_vector(4 DOWNTO 0);
BEGIN
		REG5: ENTITY work.REGN_aqf(behavioral)
			GENERIC MAP (N => 5)
			PORT MAP(	clk => clk,	en => enLFSR, D => LFSRIn,	Q => RC);
		
		GENLFSRShift:IF isINV=0 GENERATE
			LFSRShift <= RC(3 DOWNTO 0) & (RC(4) XOR RC(1));
			ROM: ENTITY work.CA_ROM(behavioral) PORT MAP(addr => pn, do => LFSRInit);
		END GENERATE GENLFSRShift;

		GENLFSRShiftInv:IF isINV=1 GENERATE
			LFSRShift <= (RC(2) XOR RC(0)) & RC(4 DOWNTO 1);
			LFSRInit <= "01100";
		END GENERATE GENLFSRShiftInv;
		
		LFSRIn <=  LFSRInit WHEN selLFSR='1' ELSE LFSRShift;
		
		GEN_V11: FOR i IN 0 TO 4 GENERATE
			V11(4-i) <= RC(4-i) XOR V_i(V-46-i);
		END GENERATE GEN_V11;

		V_o <= V_i(V-1 DOWNTO V-45) & V11 & V_i(V-51 DOWNTO 0);

END behavioral;