-------------------------------------------------------------------------------
--! @file       MC.vhd
--! @author     Ahmed Ferozpuri
--! @brief      MixColumns Component for PRIMATEs
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY MC IS
	GENERIC (C : INTEGER := 25;
				V : INTEGER := 200;
				isINV : INTEGER := 0);
	PORT(	V_i : IN  std_logic_vector(V-1 DOWNTO 0);
			V_o : OUT  std_logic_vector(V-1 DOWNTO 0)
		);
END MC;

ARCHITECTURE behavioral OF MC IS

	TYPE tType IS ARRAY (0 TO 7) OF std_logic_vector (4 DOWNTO 0); 
	TYPE cType IS ARRAY (0 TO 7) OF std_logic_vector (C-1 DOWNTO 0); 
	SIGNAL T : tType;
	SIGNAL COL : cType;
	SIGNAL VR4 : std_logic_vector (39 DOWNTO 0);
BEGIN

	VR4 <= T(0) & T(1) & T(2) & T(3) & T(4) & T(5) & T(6) & T(7);
	
	GENV_O:IF isINV=0 GENERATE
		V_o <= V_i(V-41 DOWNTO 0) & VR4;
	END GENERATE GENV_O;
	
	GENV_OINV:IF isINV=1 GENERATE
		V_o <= VR4 & V_i(V-1 DOWNTO 40);
	END GENERATE GENV_OINV;	
	
	MCP80:IF V=200 GENERATE
		GEN_CT80: FOR i IN 0 TO 7 GENERATE
			COL(i) <= 	V_i(V-1-i*5 DOWNTO V-1-i*5-4) & 
							V_i(V-1-i*5-40 DOWNTO V-1-i*5-44) & 
							V_i(V-1-i*5-80 DOWNTO V-1-i*5-84) & 
							V_i(V-1-i*5-120 DOWNTO V-1-i*5-124) & 
							V_i(V-1-i*5-160 DOWNTO V-1-i*5-164);
			Ti: 	ENTITY work.COLM(behavioral)
					GENERIC MAP (C=>C, isINV=>isINV)
					PORT MAP(C_i => COL(i), C_o => T(i));
		END GENERATE GEN_CT80;
	END GENERATE MCP80;
	
	MCP120:IF V=280 GENERATE
		GEN_CT120: FOR i IN 0 TO 7 GENERATE
			COL(i) <= 	V_i(V-1-i*5 DOWNTO V-1-i*5-4) & 
							V_i(V-1-i*5-40 DOWNTO V-1-i*5-44) & 
							V_i(V-1-i*5-80 DOWNTO V-1-i*5-84) & 
							V_i(V-1-i*5-120 DOWNTO V-1-i*5-124) & 
							V_i(V-1-i*5-160 DOWNTO V-1-i*5-164) &
							V_i(V-1-i*5-200 DOWNTO V-1-i*5-204) &
							V_i(V-1-i*5-240 DOWNTO V-1-i*5-244);
			Ti: 	ENTITY work.COLM(behavioral)
					GENERIC MAP (C=>C, isINV=>isINV)
					PORT MAP(C_i => COL(i), C_o => T(i));
		END GENERATE GEN_CT120;
	END GENERATE MCP120;
	
END behavioral;